Package: miscTools
Version: 0.6-30
Date: 2026-01-19
Title: Miscellaneous Tools and Utilities
Authors@R: c(
  person( given = "Arne", family = "Henningsen", role = c( "aut", "cre" ),
    email = "arne.henningsen@gmail.com" ),
  person( given = "Ott", family = "Toomet", role = "aut" ) )
Author: Arne Henningsen [aut, cre],
  Ott Toomet [aut]
Maintainer: Arne Henningsen <arne.henningsen@gmail.com>
Depends: R (>= 2.14.0)
Imports: digest
Suggests: Ecdat (>= 0.1-5)
Description: Miscellaneous small tools and utilities.
   Many of them facilitate the work with matrices,
   e.g. inserting rows or columns, creating symmetric matrices,
   or checking for semidefiniteness.
   Other tools facilitate the work with regression models,
   e.g. extracting the standard errors,
   obtaining the number of (estimated) parameters,
   or calculating R-squared values.
License: GPL (>= 2)
URL: https://github.com/arne-henningsen/miscTools
BugReports: https://github.com/arne-henningsen/miscTools/issues
NeedsCompilation: no
Packaged: 2026-01-19 13:56:45 UTC; gsl324
Repository: CRAN
Date/Publication: 2026-01-20 06:10:14 UTC
Built: R 4.6.0; ; 2026-04-25 16:09:26 UTC; unix

export( "checkNames" )
export( "coefTable" )
export( "colMedians" )
export( "rowMedians" )
export( "compPlot" )
export( "ddnorm" )
export( "histDens" )
export( "insertCol" )
export( "insertRow" )
export( "isSemidefinite" )
export( "margEff" )
export( "nObs" )
export( "nParam" )
export( "quasiconcavity" )
export( "quasiconvexity" )
export( "rSquared" )
export( "semidefiniteness" )
export( "stdEr" )
export( "sumKeepAttr" )
export( "summarizeDF" )
export( "symMatrix" )
export( "triang" )
export( "vecli" )
export( "vecli2m" )
export( "veclipos" )

S3method( "isSemidefinite", "default" )
S3method( "isSemidefinite", "list" )
S3method( "isSemidefinite", "matrix" )
S3method( "margEff", "default" )
S3method( "nObs", "default" )
S3method( "nObs", "lm" )
S3method( "nParam", "default" )
S3method( "nParam", "lm" )
S3method( "stdEr", "default" )
S3method( "stdEr", "lm" )

importFrom( "digest", "digest" )
importFrom( "graphics", "abline" )
importFrom( "graphics", "hist" )
importFrom( "graphics", "lines" )
importFrom( "graphics", "plot.default" )
importFrom( "stats", "coef" )
importFrom( "stats", "coefficients" )
importFrom( "stats", "density" )
importFrom( "stats", "dnorm" )
importFrom( "stats", "median" )
importFrom( "stats", "pt" )
importFrom( "stats", "vcov" )
importFrom( "stats", "sd" )
importFrom( "utils", "combn" )

library( "miscTools" )

set.seed( 123 )

# not symmetric
m1 <- matrix( rnorm( 9 ), ncol = 3 )
print( m1 )
try( semidefiniteness( m1 ) )
try( semidefiniteness( m1, method = "eigen" ) )
try( semidefiniteness( m1, positive = FALSE ) )
try( semidefiniteness( m1, positive = FALSE, method = "eigen" ) )

# positive semidefinite
m2 <- crossprod( m1 )
print( m2 )
semidefiniteness( m2 )
semidefiniteness( m2, method = "eigen" )
semidefiniteness( m2, positive = FALSE )
semidefiniteness( m2, positive = FALSE, method = "eigen" )
# negative semidefinite
semidefiniteness( -m2 )
semidefiniteness( -m2, method = "eigen" )
semidefiniteness( -m2, positive = FALSE )
semidefiniteness( -m2, positive = FALSE, method = "eigen" )

# positive semidefinite, singular
m3 <- cbind( m2, - rowSums( m2 ) )
m3 <- rbind( m3, - colSums( m3 ) )
print( m3 )
semidefiniteness( m3 )
semidefiniteness( m3, method = "eigen" )
semidefiniteness( m3, positive = FALSE )
semidefiniteness( m3, positive = FALSE, method = "eigen" )

# positive semidefinite, singular, and large numbers
m4 <- m3 * 1e6
print( m4 )
# rcond(m4)
# det(m4)
semidefiniteness( m4 )
semidefiniteness( m4, method = "eigen" )
semidefiniteness( m4, positive = FALSE )
semidefiniteness( m4, positive = FALSE, method = "eigen" )

# negative semidefinite, diagonal
m5 <- diag( -1, 4, 4 )
print( m5 )
semidefiniteness( m5 )
semidefiniteness( m5, method = "eigen" )
semidefiniteness( m5, positive = FALSE )
semidefiniteness( m5, positive = FALSE, method = "eigen" )

# negative semidefinite, singular
m6 <- matrix( -1, 4, 4 )
print( m6 )
semidefiniteness( m6 )
semidefiniteness( m6, method = "eigen" )
semidefiniteness( m6, positive = FALSE )
semidefiniteness( m6, positive = FALSE, method = "eigen" )

# negative semidefinite, diagonal
m7 <- diag( c( -1, -3 ) )
print( m7 )
semidefiniteness( m7 )
semidefiniteness( m7, method = "eigen" )
semidefiniteness( m7, positive = FALSE )
semidefiniteness( m7, positive = FALSE, method = "eigen" )

# positive semidefinite
m8 <- symMatrix( c( 2, -1, 0, 2, -1, 2 ) )
print( m8 )
semidefiniteness( m8 )
semidefiniteness( m8, method = "eigen" )
semidefiniteness( m8, positive = FALSE )
semidefiniteness( m8, positive = FALSE, method = "eigen" )

# indefinite
m9 <- symMatrix( rnorm( 6 ) )
print( m9 )
semidefiniteness( m9 )
semidefiniteness( m9, method = "eigen" )
semidefiniteness( m9, positive = FALSE )
semidefiniteness( m9, positive = FALSE, method = "eigen" )

# positive and negative semidefinite
m10 <- matrix( 0, 3, 3 )
print( m10 )
semidefiniteness( m10 )
semidefiniteness( m10, method = "eigen" )
semidefiniteness( m10, positive = FALSE )
semidefiniteness( m10, positive = FALSE, method = "eigen" )

# indefinite
m11 <- symMatrix( 1:6 )
print( m11 )
semidefiniteness( m11 )
semidefiniteness( m11, method = "eigen" )
semidefiniteness( m11, positive = FALSE )
semidefiniteness( m11, positive = FALSE, method = "eigen" )

# indefinite, singular
m12 <- cbind( m9, - rowSums( m9 ) )
m12 <- rbind( m12, - colSums( m12 ) )
print( m12 )
semidefiniteness( m12 )
semidefiniteness( m12, method = "eigen" )
semidefiniteness( m12, positive = FALSE )
semidefiniteness( m12, positive = FALSE, method = "eigen" )

# indefinite, singular, small numbers
m13 <- m12 * 1e-6
print( m13 )
semidefiniteness( m13 )
semidefiniteness( m13, method = "eigen" )
semidefiniteness( m13, positive = FALSE )
semidefiniteness( m13, positive = FALSE, method = "eigen" )

# 'large' matrix
m14 <- symMatrix( 1:( 13 * (13+1) / 2 ) )
semidefiniteness( m14 )
semidefiniteness( m14, method = "det" )
semidefiniteness( m14, method = "eigen" )

# list, one element not a matrix
ml1 <- list( m2, c( m1 ), m3, m4 )
try( semidefiniteness( ml1 ) )

# list of matrices, one non-symmetric
ml2 <- list( m2, m1, m3, m4 )
try( semidefiniteness( ml2 ) )

# list of matrices, one 'large' matrix
ml3 <- list( m2, m14, m3, m4 )
semidefiniteness( ml3 )
semidefiniteness( ml3, method = "det" )
semidefiniteness( ml3, method = "eigen" )
semidefiniteness( ml3, positive = FALSE )
semidefiniteness( ml3, positive = FALSE, method = "det" )
semidefiniteness( ml3, positive = FALSE, method = "eigen" )

## load miscTools package
library( "miscTools" )

## create a matrix
m <- matrix( 1:9, 3 )

# insert rows
print( insertRow( m, 1, 10:12 ) )
print( insertRow( m, 2, 10:12 ) )
print( insertRow( m, 3, 10:12 ) )
print( insertRow( m, 4, 10:12 ) )

# insert columns
print( insertCol( m, 1, 10:12 ) )
print( insertCol( m, 2, 10:12 ) )
print( insertCol( m, 3, 10:12 ) )
print( insertCol( m, 4, 10:12 ) )

# insert rows with name
print( insertRow( m, 1, 10:12, "R0" ) )
print( insertRow( m, 2, 10:12, "R1a" ) )
print( insertRow( m, 3, 10:12, "R2a" ) )
print( insertRow( m, 4, 10:12, "R4" ) )

# insert columns with name
print( insertCol( m, 1, 10:12, "C0" ) )
print( insertCol( m, 2, 10:12, "C1a" ) )
print( insertCol( m, 3, 10:12, "C2a" ) )
print( insertCol( m, 4, 10:12, "C4" ) )

## add row names and column names
rownames( m ) <- c( "R1", "R2", "R3" )
colnames( m ) <- c( "C1", "C2", "C3" )

# insert rows
print( insertRow( m, 1, 10:12 ) )
print( insertRow( m, 2, 10:12 ) )
print( insertRow( m, 3, 10:12 ) )
print( insertRow( m, 4, 10:12 ) )

# insert columns
print( insertCol( m, 1, 10:12 ) )
print( insertCol( m, 2, 10:12 ) )
print( insertCol( m, 3, 10:12 ) )
print( insertCol( m, 4, 10:12 ) )

# insert rows with name
print( insertRow( m, 1, 10:12, "R0" ) )
print( insertRow( m, 2, 10:12, "R1a" ) )
print( insertRow( m, 3, 10:12, "R2a" ) )
print( insertRow( m, 4, 10:12, "R4" ) )

# insert columns with name
print( insertCol( m, 1, 10:12, "C0" ) )
print( insertCol( m, 2, 10:12, "C1a" ) )
print( insertCol( m, 3, 10:12, "C2a" ) )
print( insertCol( m, 4, 10:12, "C4" ) )

# insert a row to a single-column matrix (example provided by Max Gordon)
insertRow( matrix( 1:3, ncol=1 ), 2, 4 )

# insert a column to a single-row matrix
insertCol( matrix( 1:3, nrow=1 ), 2, 4 )

## testing summarizeDF()
library( "miscTools" )
data( "cars" )

summarizeDF( cars )

summarizeDF( cars, maxLevel = 10 )

summarizeDF( cars, printValues = FALSE )

tmpFile <- file()
summarizeDF( cars, file = tmpFile )
tmpLines <- readLines( tmpFile )
close( tmpFile )
print( matrix( tmpLines, ncol = 1 ) )


data( "iris" )

summarizeDF( iris )

summarizeDF( iris, printValues = FALSE )

tmpFile <- file()
summarizeDF( iris, file = tmpFile )
tmpLines <- readLines( tmpFile )
close( tmpFile )
print( matrix( tmpLines, ncol = 1 ) )


data( "swiss" )

summarizeDF( swiss )

summarizeDF( swiss, maxLevel = 10 )

summarizeDF( swiss, printValues = FALSE )

tmpFile <- file()
summarizeDF( swiss, file = tmpFile )
tmpLines <- readLines( tmpFile )
close( tmpFile )
print( matrix( tmpLines, ncol = 1 ) )

library( "miscTools" )


## matrix
m <- matrix( 1:24, nrow = 6, ncol = 4 )

cm1 <- colMedians( m )
print( cm1 )

rm1 <- rowMedians( m )
print( rm1 )

all.equal( cm1, rowMedians( t( m ) ) )
all.equal( rm1, colMedians( t( m ) ) )


## data.frame
data( "Electricity", package = "Ecdat" )
Electricity <- Electricity[ 1:20, ]

cm2 <- colMedians( Electricity )
print( cm2 )

rm2 <- rowMedians( Electricity )
print( rm2 )

all.equal( cm2, rowMedians( t( Electricity ) ) )
all.equal( rm2, colMedians( t( Electricity ) ) )

# array (3 dimensions)
a3 <- array( 1:24, dim = c(4,3,2),
   dimnames = list( c("a","b","c","d"), c("A","B","C"), c("x","y") ) )
colMedians( a3 )
all.equal( median( a3[ , "B", "y" ] ), colMedians( a3 )[ "B", "y" ] )

# array (4 dimensions)
a4 <- array( 1:120, dim = c(5,4,3,2),
   dimnames = list( c("a","b","c","d","e"), c("A","B","C","D"),
   c("x","y","z"), c("Y","Z") ) )
colMedians( a4 )
all.equal( median( a4[ , "B", "x", "Z" ] ), colMedians( a4 )[ "B", "x", "Z" ] )

library( miscTools )

eps <- 1e-7

x <- (-40:40)/10

## standard normal distribution
ddnorm( x )
all.equal( ddnorm(x), ( dnorm( x + eps ) - dnorm( x - eps ) ) / ( 2 * eps ) )

## normal distribution (non-standard)
x <- (0:100)/10
ddnorm( x, mean = 5, sd = 2 )
all.equal( ddnorm( x, mean = 5, sd = 2),
   ( dnorm( x + eps, mean = 5, sd = 2 ) - dnorm( x - eps, mean = 5, sd = 2 ) )
      / ( 2 * eps ) )

## testing stdEr() methods
library( "miscTools" )
data( "cars" )

# lm()
lmRes <- lm(dist ~ speed, data=cars)
summary( lmRes )
round( stdEr( lmRes ), 4 )

# nls()
nlsRes <- nls( dist ~ b0 + b1 * speed^b2, start = c( b0=0, b1=1, b2=1 ),
   data = cars )
tmp <- summary( nlsRes )
tmp$convInfo$finTol <- round( tmp$convInfo$finTol, 4 )
print( tmp )
round( stdEr( nlsRes ), 3 )

## derivatives of the density function of the normal distribution
## with respect to x
ddnorm <- function( x, mean = 0, sd = 1 ) {
   deriv <- - dnorm( x = x, mean = mean, sd = sd ) * ( x - mean ) / sd^2
   return( deriv )
}

coefTable <- function( coef, stdErr, df = NULL ) {
   result <- cbind(
      coef,
      stdErr,
      coef / stdErr,
      rep( NA, length( coef ) ) )
   if( !is.null( df ) ) {
      result[ , 4 ] <- 2 * pt( abs( coef / stdErr ), df, lower.tail = FALSE )
   }
   colnames( result ) <- c( "Estimate", "Std. Error", "t value", "Pr(>|t|)" )
   if( !is.null( names( coef ) ) ) {
      rownames( result ) <- names( coef )
   }
   return( result )
}

colMedians <- function( x, na.rm = FALSE ) {

   if( is.data.frame( x ) ) {
      x <- as.matrix( x )
   }
   if( !is.array( x ) ) {
      stop( "argument 'x' must be a data.frame, matrix, or array" )
   }
   if( !is.numeric( x ) ) {
      stop( "argument 'x' must be numeric" )
   }

   result <- array( NA, dim = dim( x )[-1] )
   dimnames( result ) <- dimnames( x )[-1]

   for( i in 1:dim( x )[ 2 ] ) {
      if( length( dim( x ) ) == 2 ) {
         result[ i ] <- median( x[ , i ], na.rm = na.rm )
      } else {
         result[ slice.index( result, 1 ) == i ] <-
            colMedians( array( x[ slice.index( x, 2 ) == i ],
               dim = dim( x )[ -2 ] ), na.rm = na.rm )
      }
   }

   return( result )
}

rowMedians <- function( x, na.rm = FALSE ) {
   if( is.null( dim( x ) ) || length( dim( x ) ) != 2 ) {
      stop( "argument 'x' must be a matrix or a data.frame" )
   }
   return( colMedians( t( x ), na.rm = na.rm ) )
}

## ----- insert a column into a matrix --------------
insertCol <- function( m, c, v = NA, cName = "" ) {

   # checking the argument 'm'
   if( !inherits( m, "matrix" ) ) {
      stop( "argument 'm' must be a matrix" )
   }
   # checking the argument 'c'
   if( c == as.integer( c ) ) {
      c <- as.integer( c )
   } else {
      stop( "argument 'c' must be an integer" )
   }
   if( length( c ) != 1 ) {
      stop( "argument 'c' must be a scalar" )
   }
   if( c < 1 ) {
      stop( "argument 'c' must be positive" )
   }
   if( c > ncol( m ) + 1 ) {
      stop( "argument 'c' must not be larger than the number of columns",
         " of matrix 'm' plus one" )
   }
   # checking the argument 'cName'
   if( !is.character( cName ) ) {
      stop( "argument 'cName' must be a character string" )
   }
   if( length( cName ) != 1 ) {
      stop( "argument 'cName' must be a be a single character string" )
   }

   nr <- nrow( m )
   nc <- ncol( m )
   cNames <- colnames( m )
   if( is.null( cNames ) & cName != "" ) {
      cNames <- rep( "", nc )
   }

   if( c == 1 ) {
      m2 <- cbind( matrix( v, nrow = nr ), m )
      if( !is.null( cNames ) ) {
         colnames( m2 ) <- c( cName, cNames )
      }
   } else if( c == nc + 1 ) {
      m2 <- cbind( m, matrix( v, nrow = nr ) )
      if( !is.null( cNames ) ) {
         colnames( m2 ) <- c( cNames, cName )
      }
   } else {
      m2 <- cbind( m[ , 1:( c - 1 ), drop = FALSE ], matrix( v, nrow = nr ), 
         m[ , c:nc, drop = FALSE ] )
      if( !is.null( cNames ) ) {
         colnames( m2 ) <- c( cNames[ 1:( c - 1 ) ], cName, cNames[ c:nc ] )
      }
   }
   return( m2 )
}

## ----- insert a row into a matrix --------------
insertRow <- function( m, r, v = NA, rName = "" ) {

   # checking the argument 'm'
   if( !inherits( m, "matrix" ) ) {
      stop( "argument 'm' must be a matrix" )
   }
   # checking the argument 'r'
   if( r == as.integer( r ) ) {
      r <- as.integer( r )
   } else {
      stop( "argument 'r' must be an integer" )
   }
   if( length( r ) != 1 ) {
      stop( "argument 'r' must be a scalar" )
   }
   if( r < 1 ) {
      stop( "argument 'r' must be positive" )
   }
   if( r > nrow( m ) + 1 ) {
      stop( "argument 'r' must not be larger than the number of rows",
         " of matrix 'm' plus one" )
   }
   # checking the argument 'rName'
   if( !is.character( rName ) ) {
      stop( "argument 'rName' must be a character string" )
   }
   if( length( rName ) != 1 ) {
      stop( "argument 'rName' must be a be a single character string" )
   }

   nr <- nrow( m )
   nc <- ncol( m )
   rNames <- rownames( m )
   if( is.null( rNames ) & rName != "" ) {
      rNames <- rep( "", nr )
   }

   if( r == 1 ) {
      m2 <- rbind( matrix( v,ncol = nc ), m )
      if( !is.null( rNames ) ) {
         rownames( m2 ) <- c( rName, rNames )
      }
   } else if( r == nr + 1 ) {
      m2 <- rbind( m, matrix( v,ncol = nc ) )
      if( !is.null( rNames ) ) {
         rownames( m2 ) <- c( rNames, rName )
      }
   } else {
      m2 <- rbind( m[ 1:( r - 1 ), , drop = FALSE ], matrix( v, ncol = nc ), 
         m[ r:nr, , drop = FALSE ] )
      if( !is.null( rNames ) ) {
         rownames( m2 ) <- c( rNames[ 1:( r - 1 ) ], rName, rNames[ r:nr ] )
      }
   }
   return( m2 )
}


## ----- test a bordered Hessian for quasiconcavity ------------
quasiconcavity <- function( m, tol = .Machine$double.eps ) {

   if( is.list( m ) ) {
      result <- logical( length( m ) )
      for( t in 1:length( m ) ) {
         result[ t ] <- quasiconcavity( m[[ t ]] )
      }
   } else {
      if( !is.matrix( m ) ) {
         stop( "argument 'm' must be a matrix" )
      }
      if( nrow( m ) != ncol( m ) ) {
         stop( "argument 'm' must be a _quadratic_ matrix" )
      }
      if( nrow( m ) < 2 ) {
         stop( "a bordered Hessian has at least 2 columns/rows" )
      }
      if( m[ 1, 1 ] != 0 ) {
         stop( "element [1,1] of a bordered Hessian must be 0" )
      }

      n <- nrow( m )
      result <- TRUE
      for( i in 2:n ) {
         result <- result && det( m[ 1:i, 1:i ] ) * ( -1 )^i <= tol
      }
   }

   return( result )
}

## ----- test a bordered Hessian for quasiconvexity ------------
quasiconvexity <- function( m, tol = .Machine$double.eps ) {

   if( is.list( m ) ) {
      result <- logical( length( m ) )
      for( t in 1:length( m ) ) {
         result[ t ] <- quasiconvexity( m[[ t ]] )
      }
   } else {
      if( !is.matrix( m ) ) {
         stop( "argument 'm' must be a matrix" )
      }
      if( nrow( m ) != ncol( m ) ) {
         stop( "argument 'm' must be a _quadratic_ matrix" )
      }
      if( nrow( m ) < 2 ) {
         stop( "a bordered Hessian has at least 2 columns/rows" )
      }
      if( m[ 1, 1 ] != 0 ) {
         stop( "element [1,1] of a bordered Hessian must be 0" )
      }

      n <- nrow( m )
      result <- TRUE
      for( i in 2:n ) {
         result <- result && det( m[ 1:i, 1:i ] ) <= tol
      }
   }

   return( result )
}

## ----- Calculation of R2 value ------------
rSquared <- function( y, resid ) {
   yy <- y - matrix( mean( y ), nrow = nrow( array( y ) ) )
   r2 <- 1 -( t( resid ) %*% resid ) / ( t( yy ) %*% yy )
   return( r2 )
}


## --- creates a symmetric matrix ----
symMatrix <- function( data = NA, nrow = NULL, byrow = FALSE,
   upper = FALSE ) {

   nData <- length( data )
   if( is.null( nrow ) ) {
      nrow <- ceiling( -0.5 + ( 0.25 + 2 * nData )^0.5 - .Machine$double.eps^0.5 )
   }
   nElem <- round( nrow * ( nrow + 1 ) / 2 )
   if( nData < nElem ) {
      nRep <- nElem / nData
      data <- rep( data, ceiling( nRep ) )[ 1:nElem ]
      if( round( nRep ) != nRep ) {
         warning( "number of required values [", nElem, 
            "] is not a multiple of data length [", nData, "]" )
      }
   }
   if( nData > nElem ) {
      data <- data[ 1:nElem ]
      warning( "data length [", nData, "] is greater than number of ",
         "required values [", nElem, "]" )
   }

   result <- matrix( NA, nrow = nrow, ncol = nrow )
   if( byrow != upper ) {
      result[ upper.tri( result, diag = TRUE ) ] <- data
      result[ lower.tri( result ) ] <- t( result )[ lower.tri( result ) ]
   } else {
      result[ lower.tri( result, diag = TRUE ) ] <- data
      result[ upper.tri( result ) ] <- t( result )[ upper.tri( result ) ]
   }
   return( result )
}

## --- creates an upper triangular matrix from a vector ----
triang <- function( v, n ) {
   m <- array( 0, c( n, n ) )
   r <- ( n + 1 ) * n / 2 - dim( array( v ) )
   for( i in 1:( n - r ) ) {
      for( j in i:n ) {
         m[ i, j ] <- v[ veclipos( i, j, n ) ]
      }
   }
   return( m )
}

## creates a vector of linear indep. values from a symmetric matrix (of full rank)
vecli <- function( m ) {
   n <- dim( m )[ 1 ]
   v <- array( 0, c( ( n + 1 ) * n / 2 ) )
   for( i in 1:n ) {
      for( j in i:n ) {
         v[ veclipos( i, j, n ) ] <- m[ i, j ]
      }
   }
   return( v )
}

## creates a matrix from a vector of linear independent values
vecli2m <- function( v ) {
   nv <- dim( array( v ) )
   nm <- round( -0.5 + ( 0.25 + 2 * nv )^0.5 )
   m <- array( NA, c( nm, nm ) )
   for( i in 1:nm ) {
      for( j in 1:nm ) {
         m[ i, j ] <- v[ veclipos( i, j, nm ) ]
      }
   }
   return( m )
}

## calculation of the place of matrix elements in a vector of linear indep. values
veclipos <- function( i, j, n ) {
   pos <- n * ( n - 1 ) / 2 - ( ( n - min( i, j ) ) * ( n - min( i, j ) + 1 ) /
      2 ) + max( i, j )
   return( pos )
}

## return the sum of an array while keeping its attributes
sumKeepAttr <- function( x, keepNames = FALSE, na.rm = FALSE ) {
   xAttr <- attributes( x )
   if( !keepNames ) {
      xAttr$names <- NULL
   }
   x <- sum( x, na.rm = na.rm )
   mostattributes( x ) <- xAttr
   return( x )
}

summarizeDF <- function( dat, printValues = TRUE, maxLevel = 20,
      file = NULL, ... ) {
   if( !inherits( dat, "data.frame" ) ) {
      stop( "argument 'dat' must be a data.frame" )
   }
   if( !is.null( file ) ) {
      sink( file = file, ... )
   }
   cat( "Summary of data.frame\n" )
   cat( "number of observations:", nrow(dat), "\n" )
   cat( "number of variables:", ncol(dat), "\n" )
   cat( "MD5:", digest(dat), "\n\n" )
   
   for( i in 1:length( dat ) ) {
      cat( "variable:", names( dat )[i], "\n" )
      cat( "MD5:", digest(dat[[i]]), "\n" )
      if( isTRUE( printValues ) ) {
         if( is.numeric( dat[[i]] ) ) {
            print( cbind( summary( dat[[i]] ) ) )
            if( length( unique( dat[[i]] ) ) <= maxLevel ) {
               print( cbind( table( dat[[i]], useNA = "ifany" ) ) )
            }
         } else if( is.character( dat[[i]] ) & 
               length( unique( dat[[i]] ) ) <= maxLevel ) {
            print( cbind( table( dat[[i]], useNA = "ifany" ) ) )
         } else if( is.factor( dat[[i]] ) ) {
            if( length( levels( dat[[i]] )) <= maxLevel ) {
               print( table( dat[[i]], useNA = "ifany" ) )
            }
         } else if( is.logical( dat[[i]] ) ) {
            print( table( dat[[i]], useNA = "ifany" ) )
         }
      } else if( printValues == "mean+sd" ) {
         if( is.numeric( dat[[i]] ) ) {
            cat( "mean:", mean( dat[[i]], na.rm = TRUE ), "\n")
            cat( "sd:", sd( dat[[i]], na.rm = TRUE ), "\n" )
            if( sum( is.na( dat[[i]] ) ) > 0 ) {
               cat( "NAs:", sum( is.na( dat[[i]] ) ), "\n" )
            }
         }
      }
      cat( "\n" )
   }
   if( !is.null( file ) ) {
      sink( )
   }
}

## Return the #of parameters of model
nParam <- function(x, free=FALSE, ...)
    UseMethod("nParam")

nParam.default <- function(x, ...)
    x$param$nParam

nParam.lm <- function(x, ...)
    length(coefficients(x))

isSemidefinite <- function( m, ... )
   UseMethod( "isSemidefinite" )

isSemidefinite.default <- function( m, ... ) {
   stop( "there is currently no default method available" )
}

## ----- test positive / negative semidefiniteness
isSemidefinite.matrix <- function( m, positive = TRUE,
   tol = 100 * .Machine$double.eps,
   method = ifelse( nrow( m ) < 13, "det", "eigen" ), ... ) {

   if( !is.matrix( m ) ) {
      stop( "argument 'm' must be a matrix" )
   } else {
      if( nrow( m ) != ncol( m ) ) {
         stop( "argument 'm' or each of its elements must be a _quadratic_ matrix" )
      } else if( !isSymmetric( unname( m ), tol = 1000 * tol ) ) {
         stop( "argument 'm' must be a symmetric matrix" )
      }
      # make sure that the matrix is almost exactly symmetric
      # even if it is slightly non-symmetric 
      m <- ( m + t(m) ) / 2
      n <- nrow( m )
      if( !positive ) {
         m <- -m
      }
      if( n >= 12 && method == "det" ) {
         warning( "using method 'det' can take a very long time",
            " for matrices with more than 12 rows and columns;",
            " it is suggested to use method 'eigen' for larger matrices",
            immediate. = TRUE )
      }
      if( method == "det" ) {
         for( i in 1:n ) {
            comb <- combn( n, i )
            for( j in 1:ncol( comb ) ) {
               mat <- m[ comb[ , j ], comb[ , j ], drop = FALSE ]
               if( rcond( mat ) >= tol ) {
                  princMin <-  det( mat )
               } else {
                  princMin <- 0
               }
               if( princMin < -tol ) {
                  return( FALSE )
               }
            }
         }
         return( TRUE )
      } else if( method == "eigen" ) {
         ev <- eigen( m, only.values = TRUE )$values
         if( is.complex( ev ) ) {
            stop( "complex (non-real) eigenvalues,",
               " which could be caused by a non-symmetric matrix" )
         }
         if( all( ev > -tol ) ) {
            return( TRUE )
         } else {
            if( rcond( m ) >= tol || n == 1 ) {
               return( FALSE )
            } else {
               k <- max( 1, min( sum( abs( ev ) <= tol ), n - 1 ) )
               comb <- combn( n, n-k )
               for( j in 1:ncol( comb ) ) {
                  mm <- m[ comb[ , j ], comb[ , j ], drop = FALSE ]
                  if( !semidefiniteness( mm, tol = tol, method = method  ) ) {
                     return( FALSE )
                  }
               }
               return( TRUE )
            }
         }
      } else {
         stop( "argument 'method' must be either 'det' or 'eigen'" )
      }
   }
   stop( "internal error: please inform the maintainer",
      " of the 'miscTools' package",
      " (preferably with a reproducible example)" )
}

isSemidefinite.list <- function( m, ... ) {
   
   if( !is.list( m ) ) {
      stop( "argument 'm' must be a matrix or a list of matrices" )
   } else if( !all( sapply( m, is.matrix ) ) ) {
      stop( "all components of the list specified by argument 'm'",
         " must be matrices" )
   }
      
   result <- logical( length( m ) )
   for( t in 1:length( m ) ) {
      result[ t ] <- isSemidefinite( m[[ t ]], ... )
   }
   return( result )
}

semidefiniteness <- function( m, ... ) {
   result <- isSemidefinite( m = m, ... )
   return( result )
}

compPlot <- function( x, y, lim = NULL, ... ) {

   xyRange <- range( x, y, na.rm = TRUE, finite = TRUE )

   if( is.null( lim ) ) {
      lim <- xyRange
   } else {
      if( length( lim ) != 2 ) {
         stop( "argument 'lim' must be a vector of two elements" )
      }
      if( is.na( lim[1] ) ) {
         lim[1] <- xyRange[1]
      }
      if( is.na( lim[2] ) ) {
         lim[2] <- xyRange[2]
      }
      if( lim[1] >= lim[2] ) {
         stop( "the first element of argument 'lim' must be smaller",
            " than the second element" )
      }
      if( lim[1] > xyRange[1] |  lim[2] < xyRange[2] ) {
         warning( "some data points are outside the print area" )
      }
   }
   
   # code taken from plot.default()
   xlabel <- deparse(substitute(x))
   ylabel <- deparse(substitute(y))

   argList <- list( ... )
   
   argList$x <- x
   argList$y <- y
   argList$xlim <- lim
   argList$ylim <- lim
   if( ! "xlab" %in% names (argList) ) {
      argList$xlab <- xlabel 
   }
   if( ! "ylab" %in% names (argList) ) {
      argList$ylab <- ylabel 
   }
   
   do.call( plot.default, argList )

   abline( 0, 1 )

   invisible( xyRange )
}

histDens <- function( x, breaks = "Sturges", ... ) {
  h <- hist( x, breaks = breaks, plot = FALSE )
  d <- density( x )
  hist( x, breaks = breaks, prob = TRUE,
    ylim = c( 0, max( c( h$density, d$y ) ) ), ... )
  lines( d, lwd = 2 )
  invisible( NULL )
}

## Return #of observations for models
nObs <- function(x, ...)
    ## Number of observations for statistical models
    UseMethod("nObs")

nObs.lm <- function(x, ...)
    nrow(x$qr$qr)

nObs.default <- function(x, ...)
    x$param$nObs

checkNames <- function( testNames, allNames ) {
   inAllNames   <- testNames %in% allNames
   if( !all( inAllNames ) ) {
      stop( "object(s) '",
         paste( testNames[ !inAllNames ], collapse = "', '" ),
         "' not found." )
   }
}

### methods for extracting standard errors from the models

stdEr <- function(x, ...)
    ## Extract standard deviations from models (as coefficients)
    UseMethod("stdEr")

stdEr.default <- function(x, ...) {
   if( !isS4( x ) ) {
      if( !is.null( x$std ) ) {
         return(x$std)
      }
   }
   if(!is.null(vc <- vcov(x))) {
      s <- sqrt(diag(vc))
      names(s) <- names(coef(x))
      return(s)
   }
   return(NULL)
                           # if neither std nor vcov is defined, we return NULL...
}

stdEr.lm <- function(x, ...)
    sqrt(diag(vcov(x)))

\name{nObs}
\alias{nObs}
\alias{nObs.default}
\alias{nObs.lm}
\title{Return number of observations for statistical models}
\description{
  Returns number of observations for statistical models.  The default
  method assumes presence of a component \code{param$nObs} in \code{x}.
}
\usage{
nObs(x, \dots)
\method{nObs}{default}(x, \dots)
\method{nObs}{lm}(x, \dots)
}
\arguments{
  \item{x}{a statistical model, such as created by \code{\link{lm}}}
  \item{\dots}{further arguments for methods}
}
\details{
  This is a generic function.  The default method returns the component
  \code{x$param$nObs}.  The \code{lm}-method is based on
  qr-decomposition, in the same way as the does \code{\link{summary.lm}}.
}
\value{
  numeric, number of observations
}
\author{Ott Toomet, \email{otoomet@econ.au.dk}}

\examples{
# Construct a simple OLS regression:
x1 <- runif(100)
x2 <- runif(100)
y <- 3 + 4*x1 + 5*x2 + rnorm(100)
m <- lm(y~x1+x2)  # estimate it
nObs(m)
}
\keyword{methods}

\name{isSemidefinite}
\alias{isSemidefinite}
\alias{isSemidefinite.default}
\alias{isSemidefinite.list}
\alias{isSemidefinite.matrix}
\alias{semidefiniteness}
\title{Positive or Negative Semidefiniteness}

\description{
   Check whether a symmetric matrix is positive or negative semidefinite.
}

\usage{
isSemidefinite( m, \dots )

\method{isSemidefinite}{default}( m, \dots )

\method{isSemidefinite}{matrix}( m, positive = TRUE,
   tol = 100 * .Machine$double.eps,
   method = ifelse( nrow( m ) < 13, "det", "eigen" ), \dots )

\method{isSemidefinite}{list}( m, \dots )

semidefiniteness( m, \dots )
}

\arguments{
   \item{m}{a symmetric quadratic matrix or a list containing
      symmetric quadratic matrices.}
   \item{positive}{logical. Check for positive semidefiniteness
      (if \code{TRUE}, default)
      or for negative semidefiniteness (if \code{FALSE}).}
   \item{tol}{tolerance level (values between \code{-tol} and \code{tol}
      are considered to be zero).}
   \item{method}{method to test for semidefiniteness, either
      checking the signs of the principal minors
      (if \code{"det"}, default for matrices with up to 12 rows/columns)
      or checking the signs of the eigenvalues
      (if \code{"eigen"}, default for matrices with 13 or more rows/columns).}
   \item{\dots}{further arguments of \code{isSemidefinite.list}
      are passed to \code{isSemidefinite.matrix};.
      further arguments of \code{semidefiniteness}
      are passed to \code{isSemidefinite};
      further arguments of other functions are currently ignored.}
}

\details{
   Function \code{semidefiniteness()} passes all its arguments
   to \code{isSemidefinite()}.
   It is only kept for backward-compatibility
   and may be removed in the future.

   If argument \code{positive} is set to \code{FALSE},
   \code{isSemidefinite()} checks for negative semidefiniteness
   by checking for positive semidefiniteness
   of the negative of argument \code{m}, i.e. \code{-m}.

   If method \code{"det"} is used
   (default for matrices with up to 12 rows/columns),
   \code{isSemidefinite()} checks whether all principal minors
   (not only the leading principal minors)
   of the matrix \code{m}
   (or of the matrix \code{-m} if argument \code{positive} is \code{FALSE})
   are larger than \code{-tol}.
   Due to rounding errors,
   which are unavoidable on digital computers,
   the calculated determinants of singular (sub-)matrices
   (which should theoretically be zero)
   can considerably deviate from zero.
   In order to reduce the probability of incorrect results
   due to rounding errors,
   \code{isSemidefinite()} does not calculate the determinants
   of (sub-)matrices with reciprocal condition numbers
   smaller than argument \code{tol}
   but sets the corresponding principal minors to (exactly) zero.
   The number of principal minors of an \eqn{N \times N}{N x N} matrix is
   \eqn{\sum_{k=1}^N ( N} choose \eqn{ k )},
   which gets very large for large matrices.
   Therefore, it is not recommended to use method \code{"det"}
   for matrices with, say, more than 12 rows/columns.

   If method \code{"eigen"}
   (default for matrices with 13 or more rows/columns) is used,
   \code{isSemidefinite()} checks whether all eigenvalues
   of the matrix \code{m}
   (or of the matrix \code{-m} if argument \code{positive} is \code{FALSE})
   are larger than \code{-tol}.
   In case of a singular or nearly singular matrix,
   some eigenvalues
   that theoretically should be zero
   can considerably deviate from zero
   due to rounding errors,
   which are unavoidable on digital computers.
   \code{isSemidefinite()} uses the following procedure
   to reduce the probability of incorrectly returning \code{FALSE}
   due to rounding errors in the calculation of eigenvalues
   of singular or nearly singular matrices:
   if the reciprocal condition number of an \eqn{N \times N}{NxN} matrix
   is smaller than argument \code{tol}
   and not all of the eigenvalues of this matrix are larger than \code{-tol},
   \code{isSemidefinite()} checks
   whether all \eqn{( N} choose \eqn{ (N-k) )}
   \eqn{(N-k) \times (N-k)}{(N-k) x (N-k)} submatrices
   are positive semidefinite,
   where \eqn{k} with \eqn{0 < k < N} is the number of eigenvalues
   in the interval \code{-tol} and \code{tol}.
   If necessary, this procedure is done recursively.

   Please note that a matrix can be
   neither positive semidefinite nor negative semidefinite.
}

\value{
   \code{isSemidefinite()} and \code{semidefiniteness()}
   return a locigal value (if argument \code{m} is a matrix)
   or a logical vector (if argument \code{m} is a list)
   indicating whether the matrix (or each of the matrices)
   is positive/negative (depending on argument \code{positive})
   semidefinite.
}

\references{
   Chiang, A.C. (1984):
   \emph{Fundamental Methods of Mathematical Economics},
   3rd ed., McGraw-Hill.
   
   Gantmacher, F.R. (1959):
   \emph{The Theory of Matrices},
   Chelsea Publishing.
}

\author{Arne Henningsen}

\examples{
   # a positive semidefinite matrix
   isSemidefinite( matrix( 1, 3, 3 ))

   # a negative semidefinite matrix
   isSemidefinite( matrix(-1, 3, 3 ), positive = FALSE )

   # a matrix that is positive and negative semidefinite
   isSemidefinite( matrix( 0, 3, 3 ))
   isSemidefinite( matrix( 0, 3, 3 ), positive = FALSE )

   # a matrix that is neither positive nor negative semidefinite
   isSemidefinite( symMatrix( 1:6 ) )
   isSemidefinite( symMatrix( 1:6 ), positive = FALSE )
   
   # checking a list of matrices
   ml <- list( matrix( 1, 3, 3 ), matrix(-1, 3, 3 ), matrix( 0, 3, 3 ) )
   isSemidefinite( ml )
   isSemidefinite( ml, positive = FALSE )
}

\keyword{array}

\name{colMedians}
\alias{colMedians}
\title{Medians of Columns}

\description{
   Compute the sample medians of the columns (non-rows) of a data.frame or array.
}

\usage{
colMedians( x, na.rm = FALSE )
}

\arguments{
   \item{x}{a data.frame or array.}
   \item{na.rm}{a logical value indicating whether \code{NA} values should be
      stripped before the computation proceeds.}
}

\value{
   A vector or array of the medians of each column (non-row) of \code{x}
   with dimension \code{dim( x )[-1]}.
}

\seealso{\code{\link{rowMedians}},\code{\link{median}},\code{\link{colMeans}}.}

\author{Arne Henningsen}

\examples{
   data( "Electricity", package = "Ecdat" )
   colMedians( Electricity )

   a4 <- array( 1:120, dim = c(5,4,3,2),
      dimnames = list( c("a","b","c","d","e"), c("A","B","C","D"),
      c("x","y","z"), c("Y","Z") ) )
   colMedians( a4 )
   median( a4[ , "B", "x", "Z" ] )  # equal to
   colMedians( a4 )[ "B", "x", "Z" ]
}

\keyword{array}

\name{sumKeepAttr}
\alias{sumKeepAttr}

\title{Sum of an Array While Keeping its Attributes}

\description{
   This function returns the sum of an numeric array (e.g. vector or matrix)
   while keeping its attributes.
}

\usage{
sumKeepAttr( x, keepNames = FALSE, na.rm = FALSE )
}

\arguments{
   \item{x}{an numeric array (e.g. vector or matrix).}
   \item{keepNames}{logical. Should the name(s) of the element(s) of\code{x}
      be assigned to the returned sum?
      (only relevant if code{x} has only one element).}
   \item{na.rm}{logical. Passed to \code{\link[base]{sum}}.
      Should missing values be removed?}
}

\value{
the sum (see \code{\link[base]{sum}}).
}

\author{Arne Henningsen}

\seealso{\code{\link[base]{sum}}}

\examples{
a <- 1:10
attr( a, "min" ) <- 1
attr( a, "max" ) <- 10
sum(a)
sumKeepAttr(a)
}
\keyword{methods}

\name{summarizeDF}
\alias{summarizeDF}
\title{Summarize a data.rrame}

\description{
This function summarizes each variable that is in a data.frame.
It can be used, e.g., in an R script to write summary information 
about a data.frame into a text file that is in a version control system
so that one can see in the version control system 
whether one or more variables in the data frame have changed.
}

\usage{
summarizeDF( dat, printValues = TRUE, maxLevel = 20, file = NULL, ... )
}

\arguments{
   \item{dat}{a data.frame.}
   \item{printValues}{logical. If \code{FALSE} only MD5 checksums are returned,
      which could be desirable if the data frame contains confidential data
      that should not be included in the output.}
   \item{maxLevel}{integer. If the number of unique values in a variable 
      is less than or equal to the number specified in this argument
      (and argument \code{printValues} is \code{TRUE}),
      a frequency table is included in the output.}
   \item{file}{a character string or a writable connection naming 
      the file to write to.}
   \item{...}{further arguments forwarded to \code{sink()}
      if argument \code{file} is not \code{NULL}.}
}

\author{Arne Henningsen}
\keyword{methods}

\name{histDens}
\alias{histDens}
\title{Histogram with a Density Line}

\description{
   Plot a histrogram and add a kernel density line.
}

\usage{
histDens( x, breaks = "Sturges", ... )
}

\arguments{
   \item{x}{values of the variable.}
   \item{breaks}{passed to \code{\link[graphics]{hist}}.}
   \item{\dots}{further arguments are passed to \code{\link[graphics]{hist}}.}
}

\author{Arne Henningsen}

\examples{
   set.seed( 123 )
   x <- rnorm( 100 )
   histDens( x )
   histDens( x, 20 )
   histDens( x, 20, main = "My Title" )
}

\keyword{models}

\name{vecli2m}
\alias{vecli2m}
\title{Convert vector of linear independent values into a Matrix}

\description{
   Converts a vector into a symmetric matrix that the original vector
   contains the linear independent values of the returned symmetric
   matrix.
}

\usage{vecli2m( v )}

\arguments{
   \item{ v }{a vector.}
}

\seealso{\code{\link{vecli}}, \code{\link{veclipos}}.}

\author{Arne Henningsen}

\examples{
   v <- c( 11, 12, 13, 22, 23, 33 )
   vecli2m( v )
}

\keyword{array}

\name{nParam}
\alias{nParam}
\alias{nParam.default}
\alias{nParam.lm}
\title{Number of model parameters}
\description{
  This function returns the number of model parameters.  The default
  method returns the component \code{x$param$nParam}.
}
\usage{
nParam(x, free=FALSE, \dots)
\method{nParam}{default}(x, \dots)
\method{nParam}{lm}(x, \dots)
}

\arguments{
  \item{x}{a statistical model}
  \item{free}{logical, whether to report only the free parameters or the
    total number of parameters (default)}
  \item{\dots}{other arguments for methods}
}
\details{
  Free parameters are the parameters with no equality
  restrictions.  Some parameters may be restricted (e.g. sum of two
  probabilities may be restricted to equal unity).   In this case the
  total number of parameters may depend on the
  normalisation.
}
\value{
  Number of parameters in the model
}
\author{Ott Toomet, \email{otoomet@econ.au.dk}}
\seealso{\code{\link{nObs}} for number of observations}
\examples{
# Construct a simple OLS regression:
x1 <- runif(100)
x2 <- runif(100)
y <- 3 + 4*x1 + 5*x2 + rnorm(100)
m <- lm(y~x1+x2)  # estimate it
summary(m)
nParam(m) # you get 3
}
\keyword{methods}

\name{rSquared}
\alias{rSquared}
\title{Calculate R squared value}

\description{
   Calculate R squared value.
}

\usage{rSquared( y, resid )}

\arguments{
   \item{ y }{vector of endogenous variables}
   \item{ resid }{vector of residuals}
}

\author{Arne Henningsen}

\examples{
   data( "Electricity", package = "Ecdat" )
   reg <- lm( cost ~ q + pl + pk + pf, Electricity )
   rSquared( Electricity$cost, reg$residuals )
   summary( reg )$r.squared  # returns the same value
}

\keyword{univar}
\keyword{multivariate}
\keyword{array}

\name{insertRow}
\alias{insertRow}
\title{Insert Row into a Matrix}

\description{
   Insert a new row into a matrix.
}

\usage{insertRow( m, r, v = NA, rName = "" )}

\arguments{
   \item{ m }{matrix.}
   \item{ r }{row number where the new row should be inserted.}
   \item{ v }{optional values for the new row.}
   \item{ rName }{optional character string: the name of the new row.}
}

\value{
   a matrix with one more row than the provided matrix \code{m}.
}

\seealso{\code{\link{insertCol}}.}

\author{Arne Henningsen}

\examples{
   m <- matrix( 1:4, 2 )
   insertRow( m, 2, 5:6 )
}

\keyword{array}

\name{triang}
\alias{triang}
\title{Upper triangular matrix from a vector}

\description{
   Creates an upper triangular square matrix from a vector.
}

\usage{triang( v, n )}

\arguments{
   \item{v}{vector}
   \item{n}{desired dimension of the returned square matrix}
}

\note{
   If the vector has less elements than the upper triangular matrix,
   the last elements are set to zero.
}

\seealso{\code{\link{veclipos}}.}

\author{Arne Henningsen}

\examples{
   v <- c( 1:5 )
   triang( v, 3 )
}

\keyword{array}

\name{stdEr}
\alias{stdEr}
\alias{stdEr.default}
\alias{stdEr.lm}

\title{Standard deviations}
\description{
   Extract standard deviations from estimated models.
}
\usage{
  stdEr(x, ...)
\method{stdEr}{default}(x, \dots)
\method{stdEr}{lm}(x, \dots)
}
\arguments{
  \item{x}{a statistical model, such as created by \code{\link{lm}}}
  \item{\dots}{further arguments for methods}
}
\details{
  \code{stdEr} is a generic function with methods for objects of "lm"
  class.  The default method returns the square root of the diagonal of
  the variance-covariance matrix.
}
\value{
  numeric, the estimated standard errors of the coefficients.  
}
\author{
   Ott Toomet \email{otoomet@ut.ee}
}

\seealso{\code{\link{vcov}}, \code{\link{summary}}.}

\examples{
data(cars)
lmRes <- lm(dist ~ speed, data=cars)
stdEr( lmRes )
}

\keyword{methods}

\name{quasiconcavity}
\alias{quasiconcavity}
\alias{quasiconvexity}
\title{Test for quasiconcavity / quasiconvexity}

\description{
   Test wether a function is quasiconcave or quasiconvex.
   The bordered Hessian of this function is checked by
   \code{quasiconcavity}() or \code{quasiconvexity}().
}

\usage{
   quasiconcavity( m, tol = .Machine$double.eps )
   quasiconvexity( m, tol = .Machine$double.eps )
}

\arguments{
   \item{m}{a bordered Hessian matrix or a list containing bordered Hessian matrices}
   \item{tol}{tolerance level (values between \code{-tol} and \code{tol}
      are considered to be zero).}
}

\value{
   locigal or a logical vector (if \code{m} is a list).
}

\references{
   Chiang, A.C. (1984)
   \emph{Fundamental Methods of Mathematical Economics},
   3rd ed., McGraw-Hill.
}

\author{Arne Henningsen}

\examples{
   quasiconcavity( matrix( 0, 3, 3 ) )

   quasiconvexity( matrix( 0, 3, 3 ) )

   m <- list()
   m[[1]] <- matrix( c( 0,-1,-1, -1,-2,3, -1,3,5 ), 3, 3 )
   m[[2]] <- matrix( c( 0,1,-1, 1,-2,3, -1,3,5 ), 3, 3 )

   quasiconcavity( m )

   quasiconvexity( m )
}

\keyword{array}

\name{insertCol}
\alias{insertCol}
\title{Insert Column into a Matrix}

\description{
   Insert a new column into a matrix.
}

\usage{insertCol( m, c, v = NA, cName = "" )}

\arguments{
   \item{ m }{matrix.}
   \item{ c }{column number where the new column should be inserted.}
   \item{ v }{optional values of the new column.}
   \item{ cName }{optional character string: the name of the new column.}
}

\value{
   a matrix with one more column than the provided matrix \code{m}.
}

\seealso{\code{\link{insertRow}}.}

\author{Arne Henningsen}

\examples{
   m <- matrix( 1:4, 2 )
   insertCol( m, 2, 5:6 )
}

\keyword{array}

\name{ddnorm}
\alias{ddnorm}

\title{Derivative of the Normal Distribution's Density Function}

\description{
   This function returns the derivative(s) of the density function
   of the normal (Gaussian) distribution with respect to the quantile,
   evaluated at the quantile(s), mean(s), and standard deviation(s)
   specified by arguments \code{x}, \code{mean}, and \code{sd}, respectively.
}

\usage{
ddnorm( x, mean = 0, sd = 1 )
}

\arguments{
  \item{x}{quantile or vector of quantiles.}
  \item{mean}{mean or vector of means.}
  \item{sd}{standard deviation or vector of standard deviations.}
}

\value{
   numeric value(s): derivative(s) of the density function
   of the normal distribution with respect to the quantile
}

\author{Arne Henningsen}

\seealso{\code{\link[stats]{dnorm}}}

\examples{
ddnorm( c( -1, 0, 1 ) )
}
\keyword{methods}

\name{veclipos}
\alias{veclipos}
\title{Position in a vector of linear independent values}

\description{
   Returns the position of the [\code{i},\code{j}]th element of a symmetric
   \code{n} \eqn{\times}{x} \code{n} matrix that this element has in a vector
   of the linear independent values of the matrix.
}

\usage{veclipos( i, j, n )}

\arguments{
   \item{i}{row of the element in the matrix.}
   \item{j}{column of the element in the matrix.}
   \item{n}{dimension of the matrix.}
}

\note{
   A symmetric \code{n} \eqn{\times}{x} \code{n} matrix has n*(n+1)/2
   independent values.\cr
   The function is: n*(n-1)/2-((n-min(i,j))*(n-min(i,j)+1)/2)+max(i,j)
}

\seealso{\code{\link{vecli}}, \code{\link{vecli2m}}.}

\author{Arne Henningsen}

\examples{
   veclipos( 1, 2, 3 )  # returns: 2
}

\keyword{array}

\name{miscTools-internal}
\alias{checkNames}
% Document the following:

%%%%


\title{ Undocumented miscTools Functions }

\description{
   Undocumented miscTools Functions
}

\details{
   These are various methods or functions
   waiting for proper documentation to be written :).
}

\keyword{ internal }

\name{vecli}
\alias{vecli}
\title{Vector of linear independent values}

\description{
   Returns a vector containing the linear independent elements of a symmetric
   matrix (of full rank).
}

\usage{vecli( m )}

\arguments{
   \item{ m }{symmetric matrix}
}

\seealso{\code{\link{veclipos}}.}

\author{Arne Henningsen}

\examples{
   # a symmetric n x n matrix
   m <- cbind(c(11,12,13),c(12,22,23),c(13,23,33))
   vecli(m)  # returns: 11 12 13 22 23 33
}

\keyword{array}

\name{rowMedians}
\alias{rowMedians}
\title{Medians of Rows}

\description{
   Compute the sample medians of the rows of a data.frame or matrix.
}

\usage{
rowMedians( x, na.rm = FALSE )
}

\arguments{
   \item{x}{a data.frame or matrix.}
   \item{na.rm}{a logical value indicating whether \code{NA} values should be
      stripped before the computation proceeds.}
}

\value{
   A vector of the medians of each row of \code{x}.
}

\seealso{\code{\link{colMedians}},\code{\link{median}},\code{\link{colMeans}}.}

\author{Arne Henningsen}

\examples{
   m <- matrix( 1:12, nrow = 4 )
   rowMedians( m )
}

\keyword{array}

\name{symMatrix}
\alias{symMatrix}
\title{Symmetric Matrix}

\description{
   Create a Symmetric Matrix.
}

\usage{
   symMatrix( data = NA, nrow = NULL, byrow = FALSE,
   upper = FALSE )
}

\arguments{
   \item{data}{an optional data vector.}
   \item{nrow}{the desired number of rows and columns.}
   \item{byrow}{logical. If 'FALSE' (the default) the matrix is filled by
      columns, otherwise the matrix is filled by rows.}
   \item{upper}{logical. If 'FALSE' (the default) the lower triangular part
      of the matrix (including the diagonal) is filled, otherwise 
      the upper triangular part of the matrix is filled.}
}


\value{
   a symmetric matrix.
}

\seealso{\code{\link{matrix}}, \code{\link{lower.tri}}.}

\author{Arne Henningsen}

\examples{
   # fill the lower triangular part by columns
   symMatrix( 1:10, 4 )
   # fill the upper triangular part by columns
   symMatrix( 1:10, 4, upper = TRUE )
   # fill the lower triangular part by rows
   symMatrix( 1:10, 4, byrow = FALSE )
}

\keyword{array}

\name{compPlot}
\alias{compPlot}
\title{Scatterplot to Compare two Variables}

\description{
   Plot a scatterplot to compare two variables.
}

\usage{
compPlot( x, y, lim = NULL, ... )
}

\arguments{
   \item{x}{values of the first variable (on the X axis).}
   \item{y}{values of the second variable (on the Y axis).}
   \item{lim}{optional vector of two elements specifying the limits
      of both axes).}
   \item{\dots}{further arguments are passed to \code{\link[graphics]{plot}}.}
}

\author{Arne Henningsen}

\examples{
   set.seed( 123 )
   x <- runif( 25 )
   y <- 2 + 3 * x + rnorm( 25 )
   ols <- lm( y ~ x )
   
   compPlot( y, fitted( ols ) )
   compPlot( y, fitted( ols ), lim = c( 0, 10 ) )
   compPlot( y, fitted( ols ), pch = 20 )
   compPlot( y, fitted( ols ), xlab = "observed", ylab = "fitted" )
   compPlot( y, fitted( ols ), log = "xy" )
}

\keyword{models}

\name{coefTable}
\alias{coefTable}
\title{Coefficient Table}

\description{
   Generate Table for Coefficients, Std. Errors, t-values and P-values.
}

\usage{
   coefTable( coef, stdErr, df = NULL )
}

\arguments{
   \item{coef}{vector that contains the coefficients.}
   \item{stdErr}{vector that contains the standard errors of the coefficients.}
   \item{df}{degrees of freedom of the t-test used to calculate P-values.}
}

\value{
   a matrix with 4 columns: coefficients, standard errors, t-values
      and P-values.
   If argument \code{df} is not provided, the last column (P-values)
   is filled with \code{NA}s.
}

\author{Arne Henningsen}

\examples{
   coefTable( rnorm( 10 ), 0.5 * abs( rnorm( 10 ) ), 20 )
}

\keyword{models}

\name{margEff}
\alias{margEff}
\title{Method for Returning Marginal Effects}

\description{
Currently, this package just defines the generic function \code{margEff}
so that it can be used 
to define \code{margEff} methods for objects of specific classes
in other packages.
}

\usage{
margEff( object, \dots )
}

\arguments{
  \item{ object }{an object of which marginal effects should be calculated.}
  \item{\dots}{further arguments for methods}
}

\author{Arne Henningsen}
\keyword{methods}
